% Simulation of level discrimination with multiple procedures. Code taken largely from
% Buus' simulation of gap detecion and MML.

% MMlin50
clear all
sweetpoint=0.60;
eps=0.01;
pcatch=0.06;
obsfunction=0;

initintensitylist=[90 90 90];
initDLlist=[15 15 15];
LDLlist=[15 15 15];
%LDLlist=[.3 .6 1.2];
slopelist=[1.31 1.31 1.31];
FAlist=[0 0.05 0.1 0.15 0.2];
ntrialslist=[50];
numpsychofunctions=81;
psychometric_midpoint=[0:0.25:20];   %linspace(0+eps,20,numpsychofunctions);
numFAfunctions=4;
psychometric_FA=linspace(0,0.3,numFAfunctions);
LDLcatch=0.01;
Nruns=100;
zerocount=0;
tosscount=0;

zerocountfirst=0;
tosscountfirst=0;

if obsfunction==0
   disp('MMLIN - logistic user function');
else
   disp('MMLIN - pyes DL/DLD user function');
end

fprintf('ntrials\t slope\t FA\t LDL\t Thr     \t STD      \t meanFA     \t Zeros\t FA>20 \t Nruns\n');


for baseintensityidx=1%:length(LDLlist)
   initdiff=initDLlist(baseintensityidx);
   LDL=LDLlist(baseintensityidx);
   slope=slopelist(baseintensityidx);
   for thisFAidx=1:length(FAlist)
      thisFA=FAlist(thisFAidx);
      for numtrialsidx=1:length(ntrialslist) 
         numtrials=ntrialslist(numtrialsidx);
         numsignaltrials=round((1-pcatch)*ntrialslist(numtrialsidx));
         numcatchtrials=numtrials-numsignaltrials;
         numruns=1;
         numrunsfirst=1;
         percentyes=zeros(numpsychofunctions,numFAfunctions);
         while numruns<=Nruns
            LD(1)=initdiff;
            thiscatch=0;
            prob=zeros(numpsychofunctions,numFAfunctions);
            rspvar=rand([1 numtrials]);
            catchvar=rand([1 numtrials]);
%%%            theseFAthisrun=thisFA*2*rand([1 numtrials]);
            theseFAthisrun=thisFA*ones(1,numtrials);
            for trial=1:numtrials
               thisFAthistrial=theseFAthisrun(trial);
               if trial>1
                  LD(trial)=invlogis(sweetpoint,slope,psychometric_midpoint(ix(jx)),psychometric_FA(jx));
               end
               if LD(trial)<0
                  LD(trial)=0;
               end
               
               if (thiscatch<numcatchtrials)
                  if catchvar(trial)<pcatch
                     LD(trial)=LDLcatch;
                     thiscatch=thiscatch+1;
                  end
                  if (numtrials-trial)<=(numcatchtrials-thiscatch)
                     LD(trial)=LDLcatch;
                     thiscatch=thiscatch+1;
                  end
               end
               if obsfunction==0
                  rsp(trial)=(rspvar(trial)<=logistic(LD(trial),slope,LDL,thisFAthistrial));
               else
                  rsp(trial)=(rspvar(trial)<=pyesfunction(LD(trial),LDL,thisFAthistrial));
               end
               
               for j=1:numFAfunctions
                  if obsfunction==0
                     percentyes(:,j)=logistic(LD(trial),slope,psychometric_midpoint,psychometric_FA(j)).';
                  else
                     percentyes(:,j)=pyesfunction(LD(trial),psychometric_midpoint,psychometric_FA(j)).';
                  end
               end
               prob=prob+rsp(trial)*log(percentyes)+(1-rsp(trial))*log(1-percentyes);
               [px ix]=max(prob);	%Index of best midpoint/probability
               [pxx jx]=max(px);	%Index of best FA rate
            end % For trial=1:numtrials
            
            % FIRST ESTIMATES
            if psychometric_FA(jx)<=0.2 & psychometric_midpoint(ix(jx)) > 0
               Thrfirst(numrunsfirst)=psychometric_midpoint(ix(jx));
               falmfirst(numrunsfirst)=psychometric_FA(jx);
%               fprintf('%2.2fdB %2.2fFA ',psychometric_midpoint(ix(jx)),psychometric_FA(jx));
%               if mod(numruns,500)==0
%                  disp([int2str(numruns),' runs completed'])
%               end
               numrunsfirst=numrunsfirst+1;
            else
               %   fprintf('X%2.2f',ax(jx));
               tosscountfirst=tosscount+1;
            end
            
            if (psychometric_midpoint(ix(jx))==0)
%               fprintf('zero ');
               zerocountfirst=zerocountfirst+1;
            end
            
            
            
            
            % Re-estimation of MML using finer grain psychometric functions
            prob=zeros(numpsychofunctions,numFAfunctions);
            if psychometric_FA(jx)-(numFAfunctions*0.05)<0
               ax=[0:.05:.05*numFAfunctions];
            else
               ax=[psychometric_FA(jx)-(numFAfunctions*0.05):0.05:psychometric_FA(jx)+(numFAfunctions*0.05)];
            end
            
%            if jx>1
%               sax=psychometric_FA(jx)-0.1;
%            else
%               sax=0;
%            end
            
            %mpx=linspace(psychometric_midpoint(ix(jx))-0.13,psychometric_midpoint(ix(jx))+0.13,numpsychofunctions);
            tmpStep=(min(psychometric_midpoint)+max(psychometric_midpoint))/(10*numpsychofunctions);
            if (psychometric_midpoint(ix(jx))-(.025*(numpsychofunctions-1)/2))<0
               mpx=linspace(0,2,numpsychofunctions);
            else
               mpx=linspace(psychometric_midpoint(ix(jx))-(.025*(numpsychofunctions-1)/2),psychometric_midpoint(ix(jx))+(.025*(numpsychofunctions-1)/2),numpsychofunctions);
            end
            
%            ax=linspace(sax,sax+0.2,5);
            for trial=1:numtrials
               for j=1:numFAfunctions
                  if obsfunction==0
                     percentyes(:,j)=logistic(LD(trial),slope,mpx,ax(j)).';
                  else
                     percentyes(:,j)=pyesfunction(LD(trial),mpx,ax(j)).';
                  end
               end
               prob=prob+(rsp(trial)*log(percentyes))+((1-rsp(trial))*log(1-percentyes));
               [px ix]=max(prob);
               [pxx jx]=max(px);
            end
            
            if ax(jx)<=0.2 & mpx(ix(jx))>0
               Thr(numruns)=mpx(ix(jx));
               falm(numruns)=ax(jx);
%               if mod(numruns,500)==0
%                  disp([int2str(numruns),' runs completed'])
%               end
               numruns=numruns+1;
            else
               %   fprintf('X%2.2f',ax(jx));
               tosscount=tosscount+1;
            end
            
            if (mpx(ix(jx))==0)
%               fprintf('zero ');
               zerocount=zerocount+1;
            end
            
            
         end %while numruns<=Nruns
%         fprintf('F %d\t %2.1f\t %1.2f\t %5.2f\t %5.5f\t %5.5f\t %5.5f\t %d\t %d\t %d\n',...
%            numsignaltrials, slope, thisFA, LDL, mean(Thrfirst), std(Thrfirst), mean(falmfirst), zerocountfirst, tosscountfirst, Nruns);
         zerocountfirst=0;
         tosscountfirst=0;
         fprintf('%d\t %2.1f\t %1.2f\t %5.2f\t %5.5f\t %5.5f\t %5.5f\t %d\t %d\t %d\n',...
            numsignaltrials, slope, thisFA, LDL, mean(Thr), std(Thr), mean(falm), zerocount, tosscount, Nruns);
         zerocount=0;
         tosscount=0;
      end %for numsignaltrials=ntrialslist
   end %for thisFA=FAlist
end %for baseintensityidx=1:2
